package Data::Monad;
use strict;
use warnings;
use 5.012;

our $VERSION = "0.07";

1;

__END__

=head1 NAME

Data::Monad - An implementation of monads for Perl

=head1 DESCRIPTION

Is an implementation of monads for Perl.

This module is marked B<EXPERIMENTAL>. API could be changed without any notice.

=head1 AUTHOR

hiratara E<lt>hiratara {at} cpan.orgE<gt>

=head1 SEE ALSO

L<http://horna.org.ua/books/All_About_Monads.pdf>

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

